@@ -23,7 +23,7 @@
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#ident "$Id: depmod.c 1.8 Wed, 22 Aug 2001 15:13:56 +1000 kaos $"
+#ident "$Id: depmod.c 1.9 Sat, 22 Sep 2001 19:51:52 +1000 kaos $"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -52,6 +52,16 @@
 #define ALLOC_SYM	10000	/* Number of symbols allocated per chunk */
 #define MAX_MAP_SYM	100000	/* Maximum number of symbols in map */
 
+#ifndef __u32
+#define __u32 u_int32_t
+#endif
+#ifndef __u16
+#define __u16 u_int16_t
+#endif
+#ifndef __u8
+#define __u8 u_int8_t
+#endif
+
 typedef struct SYMBOL{
 	struct SYMBOL *next;	/* List connected via hashing */
 	struct MODULE *module;	/* Module declaring this symbol */
@@ -100,13 +110,6 @@ struct isapnp_card_id {
  * is now from kernel 2.4.0.
  */
 
-#ifndef __u16
-#define __u16 u_int16_t
-#endif
-#ifndef __u8
-#define __u8 u_int8_t
-#endif
-
 /*
  * Device table entry for "new style" table-driven USB drivers.
  * User mode code can read these tables to choose which modules to load.
@@ -182,6 +185,24 @@ struct parport_device_id {
 	ElfW(Addr) pattern;
 };
 
+/* Device tables for ieee1394 devices.
+ * Added sep. 15, 2001, Kristian Hogsberg <hogsberg@users.sf.net>.
+ * See linux/drivers/ieee1394/ieee1394_hotplug.h
+ * Although the kernel defines driver_data as void *, define it as ElfW(Addr)
+ * to make combined 32/64 code work.
+ */
+
+unsigned int ieee1394_device_id_ver;
+
+struct ieee1394_device_id {
+	__u32 match_flags;
+	__u32 vendor_id;
+	__u32 model_id;
+	__u32 specifier_id;
+	__u32 version;
+	ElfW(Addr) driver_data;
+};
+
 typedef struct MODULE {
 	char *name;
 	int resolved;
@@ -201,6 +222,8 @@ typedef struct MODULE {
 	int n_usb_device;
 	struct parport_device_id *parport_device;
 	int n_parport_device;
+	struct ieee1394_device_id *ieee1394_device;
+	int n_ieee1394_device;
 } MODULE;
 
 typedef enum SYM_STATUS {
@@ -645,6 +668,41 @@ static void extract_parport_device_id(struct obj_file *f, void *image, unsigned
 	}
 }
 
+/*
+ *	Extract any ieee1394_device_table from the module.
+ *	Note: assumes same machine and arch for depmod and module.
+ */
+static void extract_ieee1394_device_id(struct obj_file *f, void *image, unsigned long m_size, MODULE *mod)
+{
+	struct ieee1394_device_id ieee1394_device, *latest;
+	ElfW(Addr) ref_ieee1394, ref_ref_ieee1394;
+	unsigned tgt_long ieee1394_device_size;
+	ref_ieee1394 = obj_symbol_final_value(f, obj_find_symbol(f, "__module_ieee1394_device_size"));
+	if (!in_range(f, m_size, ref_ieee1394, sizeof(ieee1394_device_size)))
+		return;
+	memcpy(&ieee1394_device_size, (char *)image + ref_ieee1394 - f->baseaddr, sizeof(ieee1394_device_size));
+	ref_ref_ieee1394 = obj_symbol_final_value(f, obj_find_symbol(f, "__module_ieee1394_device_table"));
+	if (!in_range(f, m_size, ref_ref_ieee1394, sizeof(ref_ieee1394)))
+		return;
+	memcpy(&ref_ieee1394, (char *)image + ref_ref_ieee1394 - f->baseaddr, sizeof(ref_ieee1394));
+	extract_version(f, image, m_size, "ieee1394_device", &ieee1394_device_id_ver, 1, 1);
+	if (ieee1394_device_size != sizeof(ieee1394_device)) {
+		error("Unexpected value (%" tgt_long_fmt "d) in '%s' for ieee1394_device_size%s",
+		ieee1394_device_size, f->filename, has_kernel_changed);
+		exit(-1);
+	}
+	while (in_range(f, m_size, ref_ieee1394, ieee1394_device_size)) {
+		memset(&ieee1394_device, 0, sizeof(ieee1394_device));
+		memcpy(&ieee1394_device, (char *)image + ref_ieee1394 - f->baseaddr, ieee1394_device_size);
+		ref_ieee1394 += ieee1394_device_size;
+		if (ieee1394_device.match_flags == 0)
+			break;
+		mod->ieee1394_device = xrealloc(mod->ieee1394_device, ++(mod->n_ieee1394_device)*sizeof(*(mod->ieee1394_device)));
+		latest = mod->ieee1394_device + mod->n_ieee1394_device-1;
+		*latest = ieee1394_device;
+	}
+}
+
 /*
  *	Read the symbols in an object and register them in the symbol table.
  *	Return -1 if there is an error.
@@ -827,6 +885,7 @@ static int loadobj(const char *objname, int ignore_suffix)
 	extract_isapnp_card_id(f, image, m_size, mod);
 	extract_usb_device_id(f, image, m_size, mod);
 	extract_parport_device_id(f, image, m_size, mod);
+	extract_ieee1394_device_id(f, image, m_size, mod);
 	free(image);
 
 	obj_free(f);
@@ -1025,6 +1084,7 @@ static int prtdepend(char *base_dir, int nflag)
 	FILE *isapnpmap = stdout;
 	FILE *usbmap = stdout;
 	FILE *parportmap = stdout;
+	FILE *ieee1394map = stdout;
 	MODULE **tbdep;
 	MODULE *ptmod;
 	int i;
@@ -1038,6 +1098,7 @@ static int prtdepend(char *base_dir, int nflag)
 		isapnpmap = gen_file_open(gen_file+GEN_ISAPNPMAPFILE);
 		usbmap = gen_file_open(gen_file+GEN_USBMAPFILE);
 		parportmap = gen_file_open(gen_file+GEN_PARPORTMAPFILE);
+		ieee1394map = gen_file_open(gen_file+GEN_IEEE1394MAPFILE);
 	}
 
 	skipchars = strlen(base_dir);
@@ -1216,6 +1277,24 @@ static int prtdepend(char *base_dir, int nflag)
 		}
 	}
 
+	ptmod = modules;
+	fprintf(ieee1394map, "# ieee1394 module    match_flags vendor_id model_id specifier_id version\n");
+	for (i = 0; i < n_modules; i++, ptmod++) {
+		int j;
+		struct ieee1394_device_id *ieee1394_device = ptmod->ieee1394_device;
+		if (!ptmod->n_ieee1394_device)
+			continue;
+		for (j = 0; j < ptmod->n_ieee1394_device; j++, ieee1394_device++) {
+			fprintf(ieee1394map, "%-20s 0x%08x  0x%06x  0x%06x 0x%06x     0x%06x\n", 
+				shortname(ptmod->name),
+				ieee1394_device->match_flags,
+				ieee1394_device->vendor_id,
+				ieee1394_device->model_id,
+				ieee1394_device->specifier_id,
+				ieee1394_device->version);
+		}
+	}
+
 	if (generic_string != stdout)
 		fclose(generic_string);
 	if (pcimap != stdout)
@@ -1226,6 +1305,8 @@ static int prtdepend(char *base_dir, int nflag)
 		fclose(usbmap);
 	if (parportmap != stdout)
 		fclose(parportmap);
+	if (ieee1394map != stdout)
+		fclose(ieee1394map);
 	/* Close depfile last, it's timestamp is critical */
 	if (dep != stdout)
 		fclose(dep);
