@@ -24,7 +24,7 @@
 #ifndef MODUTILS_OBJ_H
 #define MODUTILS_OBJ_H 1
 
-#ident "$Id: obj.h 1.4 Thu, 26 Jul 2001 12:52:15 +1000 kaos $"
+#ident "$Id: obj.h 1.5 Sat, 22 Sep 2001 20:06:32 +1000 kaos $"
 
 /* The relocatable object is manipulated using elfin types.  */
 
@@ -200,7 +200,8 @@ void obj_insert_section_load_order (struct obj_file *f,
 struct obj_section *obj_create_alloced_section (struct obj_file *f,
 						const char *name,
 						unsigned long align,
-						unsigned long size);
+						unsigned long size,
+						unsigned long flags);
 
 struct obj_section *obj_create_alloced_section_first (struct obj_file *f,
 						      const char *name,
