@@ -31,9 +31,11 @@
  *   Keith Owens <kaos@ocs.com.au> December 1999.
  * Print " persistent" for persistent parameters.
  *   Keith Owens <kaos@ocs.com.au> November 2000.
+ * License support.
+ *   Keith Owens <kaos@ocs.com.au> September 2001.
  */
 
-#ident "$Id: modinfo.c 1.6 Wed, 28 Mar 2001 20:49:06 +1000 kaos $"
+#ident "$Id: modinfo.c 1.7 Tue, 25 Sep 2001 15:01:28 +1000 kaos $"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -249,6 +251,11 @@ static char *format_query_string(struct obj_file *f, const char *format)
 				&out_str, last_char - out_str, 0);
 			break;
 
+		case 'l':
+			(void) append_modinfo_tag(f, "license", "<none>",
+				&out_str, last_char - out_str, 1);
+			break;
+
 		case '{':{
 				char tag[128], *end = strchr(in_str, '}');
 				int multi_line;
@@ -267,7 +274,8 @@ static char *format_query_string(struct obj_file *f, const char *format)
 				strncpy(tag, in_str, end - in_str);
 				tag[end - in_str] = '\0';
 				multi_line = strcmp(tag, "author") == 0 ||
-					     strcmp(tag, "description") == 0;
+					     strcmp(tag, "description") == 0 ||
+					     strcmp(tag, "license") == 0;
 
 				/* Append the tag's value if it exists. */
 				append_modinfo_tag(f, tag, "<none>", &out_str, last_char - out_str, multi_line);
@@ -479,6 +487,7 @@ void modinfo_usage(void)
 	printf("usage: modinfo <options> <module>\n");
 	printf("  -a, --author            display module author\n");
 	printf("  -d, --description       display module description\n");
+	printf("  -l, --license           display module license\n");
 	printf("  -n, --filename          display module filename\n");
 	printf("  -f <str>\n");
 	printf("      --format <str>      print the <query-string>\n");
@@ -499,6 +508,7 @@ int main(int argc, char *argv[])
 	{
 		{"author", 0, 0, 'a'},
 		{"description", 0, 0, 'd'},
+		{"license", 0, 0, 'l'},
 		{"filename", 0, 0, 'n'},
 		{"format", required_argument, 0, 'f'},
 		{"parameters", 0, 0, 'p'},
@@ -507,9 +517,9 @@ int main(int argc, char *argv[])
 		{0, 0, 0, 0}
 	};
 	int do_parameters = 1, opt;
-	char *fmtstr = "filename:    %n\ndescription: %d\nauthor:      %a\n";
+	char *fmtstr = "filename:    %n\ndescription: %d\nauthor:      %a\nlicense:     %l\n";
 
-	while ((opt = getopt_long(argc, argv, "adnq:pf:Vh", long_opts, NULL)) != EOF)
+	while ((opt = getopt_long(argc, argv, "adlnq:pf:Vh", long_opts, NULL)) != EOF)
 		switch (opt) {
 		case 'a':
 			fmtstr = "%a\n";
@@ -519,6 +529,10 @@ int main(int argc, char *argv[])
 			fmtstr = "%d\n";
 			do_parameters = 0;
 			break;
+		case 'l':
+			fmtstr = "%l\n";
+			do_parameters = 0;
+			break;
 		case 'n':
 			fmtstr = "%n\n";
 			do_parameters = 0;
