@@ -19,7 +19,7 @@
    along with this program; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
-#ident "$Id: obj_alpha.c 1.1 Fri, 05 Jan 2001 12:45:19 +1100 kaos $"
+#ident "$Id: obj_alpha.c 1.2 Sat, 22 Sep 2001 20:06:32 +1000 kaos $"
 
 #include <string.h>
 #include <assert.h>
@@ -28,6 +28,12 @@
 #include <obj.h>
 #include <util.h>
 
+
+/* This relocation got renamed, and the change hasn't propagated yet.  */
+#ifndef R_ALPHA_GPREL16
+#define R_ALPHA_GPREL16  R_ALPHA_IMMED_GP_16
+#endif
+
 /*======================================================================*/
 
 struct alpha_got_entry
@@ -100,6 +106,7 @@ arch_apply_relocation (struct obj_file *f,
 
   unsigned long *lloc = (unsigned long *)(targsec->contents + rel->r_offset);
   unsigned int *iloc = (unsigned int *)lloc;
+  unsigned short *sloc = (unsigned short *)lloc;
   Elf64_Addr dot = targsec->header.sh_addr + rel->r_offset;
   Elf64_Addr gp = af->got->header.sh_addr + 0x8000;
 
@@ -115,6 +122,30 @@ arch_apply_relocation (struct obj_file *f,
       *lloc += v;
       break;
 
+    case R_ALPHA_GPREL16:
+      v -= gp;
+      if ((Elf64_Sxword)v > 0x7fff
+	  || (Elf64_Sxword)v < -(Elf64_Sxword)0x8000)
+	ret = obj_reloc_overflow;
+      *sloc = v;
+      break;
+
+    case R_ALPHA_GPRELLOW:
+      /* GPRELLOW does not overflow.  Errors are seen in the
+	 corresponding GPRELHIGH.  */
+      v -= gp;
+      *sloc = v;
+      break;
+
+    case R_ALPHA_GPRELHIGH:
+      v -= gp;
+      v = ((Elf64_Sxword)v >> 16) + ((v >> 15) & 1);
+      if ((Elf64_Sxword)v > 0x7fff
+	  || (Elf64_Sxword)v < -(Elf64_Sxword)0x8000)
+	ret = obj_reloc_overflow;
+      *sloc = v;
+      break;
+
     case R_ALPHA_GPREL32:
       v -= gp;
       if ((Elf64_Sxword)v > 0x7fffffff
@@ -138,7 +169,7 @@ arch_apply_relocation (struct obj_file *f,
 	    gotent->reloc_done = 1;
 	  }
 
-	*iloc = (*iloc & ~0xffff) | ((gotent->offset - 0x8000) & 0xffff);
+	*sloc = gotent->offset - 0x8000;
       }
     break;
 
@@ -270,9 +301,10 @@ arch_create_got (struct obj_file *f)
     }
 
   /* We always want a .got section so that we always have a GP for
-     use with GPDISP and GPREL32 relocs.  Besides, if the section
+     use with GPDISP and GPREL relocs.  Besides, if the section
      is empty we don't use up space anyway.  */
-  af->got = obj_create_alloced_section(&af->root, ".got", 8, offset);
+  af->got = obj_create_alloced_section(&af->root, ".got", 8, offset,
+				       SHF_WRITE | SHF_ALPHA_GPREL);
 
   return 1;
 }
