@@ -19,7 +19,7 @@
    along with this program; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
-#ident "$Id: obj_i386.c 1.1 Fri, 05 Jan 2001 12:45:19 +1100 kaos $"
+#ident "$Id: obj_i386.c 1.2 Sat, 22 Sep 2001 20:06:32 +1000 kaos $"
 
 #include <string.h>
 #include <assert.h>
@@ -216,7 +216,8 @@ arch_create_got (struct obj_file *f)
     }
 
   if (offset > 0 || gotneeded)
-    ifile->got = obj_create_alloced_section(&ifile->root, ".got", 4, offset);
+    ifile->got = obj_create_alloced_section(&ifile->root, ".got", 4, offset,
+					    SHF_WRITE);
 
   return 1;
 }
