@@ -19,7 +19,7 @@
    along with this program; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
-#ident "$Id: obj_reloc.c 1.2 Wed, 22 Aug 2001 15:19:15 +1000 kaos $"
+#ident "$Id: obj_reloc.c 1.3 Sat, 22 Sep 2001 20:06:32 +1000 kaos $"
 
 #include <string.h>
 #include <assert.h>
@@ -48,7 +48,7 @@ obj_string_patch(struct obj_file *f, int secidx, ElfW(Addr) offset,
   strsec = obj_find_section(f, ".kstrtab");
   if (strsec == NULL)
     {
-      strsec = obj_create_alloced_section(f, ".kstrtab", 1, len);
+      strsec = obj_create_alloced_section(f, ".kstrtab", 1, len, 0);
       p->string_offset = 0;
       loc = strsec->contents;
     }
@@ -294,6 +294,9 @@ obj_relocate (struct obj_file *f, ElfW(Addr) base)
       targsec = f->sections[relsec->header.sh_info];
       strsec = f->sections[symsec->header.sh_link];
 
+      if (!(targsec->header.sh_flags & SHF_ALLOC))
+	continue;
+
       rel = (ElfW(RelM) *)relsec->contents;
       relend = rel + (relsec->header.sh_size / sizeof(ElfW(RelM)));
       symtab = (ElfW(Sym) *)symsec->contents;
@@ -371,17 +374,8 @@ obj_relocate (struct obj_file *f, ElfW(Addr) base)
 	      errmsg = "Modules compiled with -mconstant-gp cannot be loaded";
 	      goto bad_reloc;
 	    bad_reloc:
-	      if (extsym)
-		{
-		  error("%s of type %ld for %s", errmsg,
-			(long)ELFW(R_TYPE)(rel->r_info),
-			strtab + extsym->st_name);
-		}
-	      else
-		{
-		  error("%s of type %ld", errmsg,
-			(long)ELFW(R_TYPE)(rel->r_info));
-		}
+	      error("%s of type %ld for %s", errmsg,
+		    (long)ELFW(R_TYPE)(rel->r_info), intsym->name);
 	      ret = 0;
 	      break;
 	    }
