@@ -23,7 +23,7 @@
    along with this program; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
-#ident "$Id: obj_s390.c 1.3 Wed, 15 Aug 2001 21:44:17 +1000 kaos $"
+#ident "$Id: obj_s390.c 1.4 Sat, 22 Sep 2001 20:06:32 +1000 kaos $"
 
 #include <string.h>
 #include <assert.h>
@@ -304,7 +304,7 @@ arch_create_got (struct obj_file *f)
 
     gotsec = obj_find_section(f, ".got");
     if (gotsec == NULL)
-      gotsec = obj_create_alloced_section(f, ".got", 4, got_offset);
+      gotsec = obj_create_alloced_section(f, ".got", 4, got_offset, SHF_WRITE);
     else
       obj_extend_section(gotsec, got_offset);
     gotsym = obj_add_symbol(f, "_GLOBAL_OFFSET_TABLE_", -1,
@@ -315,7 +315,8 @@ arch_create_got (struct obj_file *f)
   }
 
   if (plt_offset > 0)
-    ifile->plt = obj_create_alloced_section(f, ".plt", 4, plt_offset);
+    ifile->plt = obj_create_alloced_section(f, ".plt", 4, plt_offset,
+					    SHF_WRITE);
 
   return 1;
 }
